// Package proxy...
//
// Description : 正向代理的实现
//
// Author : go_developer@163.com<白茶清欢>
//
// Date : 2021-08-06 2:07 下午
package proxy

import (
	"bytes"
	"compress/gzip"
	"fmt"
	"io"
	"io/ioutil"
	"net"
	"net/http"
	"strings"
)

// Forward 正向代理的实现
//
// Author : go_developer@163.com<白茶清欢>
//
// Date : 2:08 下午 2021/8/6
func Forward(rw http.ResponseWriter, req *http.Request, serverConfig *Server) {
	fmt.Printf("Received request %s %s %s\n", req.Method, req.Host, req.RemoteAddr)

	transport := http.DefaultTransport

	// step 1
	outReq := new(http.Request)
	*outReq = *req // this only does shallow copies of maps

	if clientIP, _, err := net.SplitHostPort(req.RemoteAddr); err == nil {
		if prior, ok := outReq.Header["X-Forwarded-For"]; ok {
			clientIP = strings.Join(prior, ", ") + ", " + clientIP
		}
		outReq.Header.Set("X-Forwarded-For", clientIP)
	}

	// 写入重写的请求Header
	for k, v := range serverConfig.RewriteRequestHeader {
		outReq.Header.Set(k, v)
	}

	// 重写请求地址
	outReq.Host = serverConfig.Host
	outReq.URL.Path = serverConfig.URI
	outReq.URL.Scheme = serverConfig.Scheme
	outReq.URL.Host = serverConfig.Host

	// step 2
	res, err := transport.RoundTrip(outReq)
	if err != nil {
		rw.WriteHeader(http.StatusBadGateway)
		return
	}

	// step 3
	for key, value := range res.Header {
		for _, v := range value {
			if strings.ToLower(key) == "content-encoding" {
				continue
			}
			rw.Header().Add(key, v)
		}
	}

	rw.WriteHeader(res.StatusCode)

	// 重写请求header
	for k, v := range serverConfig.RewriteResponseHeader {
		rw.Header().Set(k, v)
	}

	// 重写响应数据
	if !strings.Contains(strings.ToLower(res.Header.Get("Content-Type")), "application/json") || nil == serverConfig.RewriteResponseData || len(serverConfig.RewriteResponseData) == 0 {
		io.Copy(rw, res.Body)
	} else {
		var responseData []byte
		var gzipData io.Reader
		gzipData, err = gzip.NewReader(res.Body)
		responseData, err = ioutil.ReadAll(gzipData)
		fmt.Println(string(responseData), err)

		bytesBuffer := bytes.NewReader([]byte(`{"data":{"permission":true}}`))
		io.Copy(rw, bytesBuffer)
	}
	res.Body.Close()
}
